r"""The ID:s of components used for identification and session state management."""

# =====================================================================================
# Register form
# =====================================================================================

BP_REGISTER_FORM = 'bp-register-form'
BP_REGISTER_FORM_USERNAME_TEXT_INPUT = 'bp-register-form-username-text-input'
BP_REGISTER_FORM_DISPLAYNAME_TEXT_INPUT = 'bp-register-form-displayname-text-input'
BP_REGISTER_FORM_ALIASES_TEXT_INPUT = 'bp-register-form-aliases-text-input'
BP_REGISTER_FORM_SUBMIT_BUTTON = 'bp-register-form-submit-button'

# =====================================================================================
# Sign in form
# =====================================================================================

BP_SIGN_IN_FORM_ALIAS_TEXT_INPUT = 'bp-sign-in-form-alias-text-input'
BP_SIGN_IN_FORM_SUBMIT_BUTTON = 'bp-sign-in-form-submit-button'
